Class: Extface::Driver::StarScp700
- Inherits:
-
StarTsp200
- Object
- Driver::GenericPos
- StarTsp200
- Extface::Driver::StarScp700
- Defined in:
- app/models/extface/driver/star_scp700.rb
Constant Summary collapse
- NAME =
'Star SCP700 (Receipt only)'.freeze
- GROUP =
Extface::PRINT_DRIVER
- DEVELOPMENT =
driver is not ready for production (not passing all tests or has major bugs)
true- CAN_RECEСVE_DATA =
pull from server
true- CAN_TRANSMIT_DATA =
push to server
true- RAW =
Select driver features
true- PRINT =
responds to #push(data) and #pull
true- FISCAL =
POS, slip printers
false- REPORT =
cash registers, fiscal printers
false
Instance Method Summary collapse
-
#autocut(partial = true) ⇒ Object
only transmit data that must be parsed by handler, CDR, report devices.
Instance Method Details
#autocut(partial = true) ⇒ Object
only transmit data that must be parsed by handler, CDR, report devices
17 18 19 20 |
# File 'app/models/extface/driver/star_scp700.rb', line 17 def autocut(partial = true) print "\r\n\r\n\r\n" print partial ? "\x1B\x64\x31" : "\x1B\x64\x30" end |